# squidGuard.pm - The Object class that provides the squidGuard Application Object.
# Created by James A. Pattie, 07/11/2000.

package Portal::squidGuard;
# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;
use Portal::squidGuard::Language;
use DBIWrapper;
use Portal::SessionHandler;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.0.02';

# global variables
my %states = ( Main => "Main Menu", Server => "Server Module", Rules => "Rules Module", );

# instantiates an instance of an Object
# takes: input, langObj, applicationObj, cookieObj, configObj, appObj, userObj, companyObj, portalSession, browserType, browserVersion, browserMaker, authObj
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( input => undef, langObj => undef, applicationObj => undef, cookieObj => undef, configObj => undef, appObj => undef, userObj => undef,
               companyObj => undef, portalSession => undef, browserType => "HTML", browserVersion => "4.0", browserMaker => "", authObj => undef, @_ );
  my $errStr = "Portal::squidGuard->new()  - Error!<br>\n";

  $self->{applicationObj} = $args{applicationObj};  # The Portal::Application module.
  $self->{input} = $args{'input'};
  $self->{cookieObj} = $args{cookieObj};
  $self->{configObj} = $args{configObj};
  $self->{appObj} = $args{appObj};  # info about the application being run.
  $self->{userObj} = $args{userObj};
  $self->{companyObj} = $args{companyObj};
  $self->{portalSession} = $args{portalSession};
  $self->{browserType} = $args{browserType};
  $self->{browserVersion} = $args{browserVersion};
  $self->{browserMaker} = $args{browserMaker};
  $self->{authObj} = $args{authObj};

  my $lang = $args{langObj}->{lang};  # get the language to use from the passed in language object (Portal)

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  $self->{portalDB} = $self->{applicationObj}->{portalDB};

  # get our Language Object
  my $langObj = Portal::squidGuard::Language->new(lang => $lang);
  if ($langObj->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $langObj->errorMessage);

    return $self;
  }

  $self->{langObj} = $langObj;

  # setup the Session (database location, name, user, password, etc.)
  my $appSessions = Portal::SessionHandler->new(dbiw => $self->{portalDB}, dbType => $self->{configObj}->{dbType}, sessionObj => $self->{portalSession},
                                           cookieDomain => $self->{configObj}->{cookieDomain}, cookieLife => $self->{configObj}->{cookieLife}, sessionType => $self->{configObj}->{sessionType},
                                           fileDir => $self->{configObj}->{sessionDirectory}, lockDir => $self->{configObj}->{sessionLockDir});
  if ($appSessions->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $appSessions->errorMessage);
    return $self;
  }
  $self->{appSessions} = $appSessions;

  # see if our app has a session started yet, if not create one.
  my $firstTime = 0;
  if (!exists $appSessions->{apps}->{squidGuard})
  {
    $firstTime = 1;
    $self->{appSessions}->makeSession(name => "squidGuard");
    if ($self->{appSessions}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{appSessions}->errorMessage);
      return $self;
    }

    # use the companyObj and userObj to get the info for this application as far as
    # the database server, port, type, user, password and database name to connect to.

    my $companyAppObj = $self->{applicationObj}->getCompanyAppInfo(companyId => $self->{companyObj}->id, appId => $self->{appObj}->id);
    if ($self->{applicationObj}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{applicationObj}->errorMessage);
      return $self;
    }
    if (!defined $companyAppObj)
    {
      $self->setError(errorString => $errStr . "Application = 'squidGuard' not assigned to Company = '$self->{companyObj}->{name}'!<br>\n");
      return $self;
    }

    # now we need to store the database info in the session.
    # in this program I am working with the Portal database and not a database per company scenario, so I need to point the database Host to the Portal Database Server
    # which may not be the same thing as the server defined in my Company App Object.  The application is still being run on the server defined in the Company App Object
    # we are just going across the network for the database we are going to use.
    $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbHost}   = $self->{configObj}->{dbHost}; # $companyAppObj->server;
    $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbName}   = $companyAppObj->dbName;
    $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbType}   = $companyAppObj->dbType;
    $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbPort}   = $companyAppObj->dbPort;
    $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbUser}   = $self->{configObj}->dbUser;
    $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbPasswd} = $self->{configObj}->dbPasswd;

    # store the companyAppObj in the session for this application so later states can get access to it.
    $self->{appSessions}->{apps}->{squidGuard}->{store}->{companyAppObj} = $companyAppObj;

    $self->{appSessions}->{apps}->{squidGuard}->{store}->{windows} = {};  # The hash that keeps track of what windows are open.

    $self->{appSessions}->{apps}->{squidGuard}->{store}->{okToLogout} = 1;
  }

  # use the database info from the session to connect to the database and store the handle internally.
  my $dbHost = $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbHost};
  my $dbName = $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbName};
  my $dbType = $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbType};
  my $dbPort = $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbPort};
  my $dbUser = $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbUser};
  my $dbPasswd = $self->{appSessions}->{apps}->{squidGuard}->{store}->{dbPasswd};

  my $dbHandle = DBIWrapper->new(dbType => $dbType, dbPort => $dbPort, dbHost => $dbHost, dbName => $dbName, dbUser => $dbUser, dbPasswd => $dbPasswd);
  if ($dbHandle->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $dbHandle->errorMessage);
    if ($firstTime)
    {
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove this session.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
    }
    return $self;
  }
  $self->{dbHandle} = $dbHandle;

  $self->{firstTime} = $firstTime;

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard->isValid() - Error!<br>\n";

  # do validation code here.
  if (not defined $self->{input})
  {
    $errorString .= "Could not get input hash.<br>\n";
  }
  if (not defined $self->{portalSession})
  {
    $errorString .= "portalSession not defined!<br>\n";
  }
  elsif (!$self->{portalSession}->isValid)
  {
    $errorString .= "portalSession is not valid!<br>\n";
  }
  if (not defined $self->{appObj})
  {
    $errorString .= "appObj not defined!<br>\n";
  }
  elsif (!$self->{appObj}->isValid)
  {
    $errorString .= "appObj is not valid!<br>\n";
  }
  if (not defined $self->{applicationObj})
  {
    $errorString .= "applicationObj not defined!<br>\n";
  }
  elsif (!$self->{applicationObj}->isValid)
  {
    $errorString .= "applicationObj is not valid!<br>\n";
  }
  if (not defined $self->{userObj})
  {
    $errorString .= "userObj not defined!<br>\n";
  }
  elsif (!$self->{userObj}->isValid)
  {
    $errorString .= "userObj is not valid!<br>\n";
  }
  if (not defined $self->{companyObj})
  {
    $errorString .= "companyObj not defined!<br>\n";
  }
  elsif (!$self->{companyObj}->isValid)
  {
    $errorString .= "companyObj is not valid!<br>\n";
  }
  if (not defined $self->{authObj})
  {
    $errorString .= "authObj not defined!<br>\n";
  }
  elsif (!$self->{authObj}->isValid)
  {
    $errorString .= "authObj is not valid!<br>\n";
  }
  if (not defined $self->{cookieObj})
  {
    $errorString .= "cookieObj not defined!<br>\n";
  }
  if (not defined $self->{configObj})
  {
    $errorString .= "configObj not defined!<br>\n";
  }
  if ($self->{browserType} !~ /^(HTML|HDML|WML)$/)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }
  if (length $self->{browserVersion} == 0)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = 1;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

# run - The main entry point for this application.
# takes: state, command
# returns: The HTMLObject document to display or undef if an error.
sub run
{
  my $self = shift;
  my %args = ( state => "", command => "", @_ );
  my $state = $args{state};
  my $command = $args{command};
  my $dbHandle = $self->dbHandle;
  my $doc = undef;
  my $errStr = "Portal::squidGuard->run()  - Error!<br>\n";
  my $phrases = $self->langObj->phraseObj->phrases;

  if (length $state == 0)
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{stateNeeded}));
    if ($self->{firstTime})
    {
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
    }
    return $doc;
  }
  if (length $command == 0)
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{commandNeeded}));
    if ($self->{firstTime})
    {
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
    }
    return $doc;
  }

  # validate the state is known.
  if (!exists $states{$state})
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{stateNotKnown}, $state));
    if ($self->{firstTime})
    {
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
    }
    return $doc;
  }

  # make sure the user has this application assigned to them that is attempting to run it
  if ($self->{firstTime})
  {
    my $result = $self->{applicationObj}->isAppAssignedToUser(appName => "squidGuard", userId => $self->{userObj}->{id});
    if ($self->{applicationObj}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{applicationObj}->errorMessage);
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
      return $doc;
    }
    if ($result == 0)
    {
      $self->setError(errorString => $errStr . "You do not have this application assigned to you!<br>\n");
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
      return $doc;
    }
  }

  my $stateName = "Portal::squidGuard::$state";
  eval "use $stateName;";
  if ($@)
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{stateEvalFailed}, $stateName, $@));
    if ($self->{firstTime})
    {
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
    }
    return $doc;
  }

  # make a logObj  # The Database for this Application better have a log_tb defined!
  my $logObj = Portal::Log->new(dbiw => $dbHandle);
  if ($logObj->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $logObj->errorMessage);
    return $doc;
  }

  if ($self->{firstTime})  # make the log entry (3) to indicate this app has been launched.
  {
    my $logEntry = Portal::Objects::LogEntry->new(action => 3, ipAddress => (exists $ENV{REMOTE_ADDR} ? $ENV{REMOTE_ADDR} : '127.0.0.1'), extraInfo => "squidGuard", userId => $self->{userObj}->{id});
    if ($logEntry->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $logEntry->errorMessage);
      return undef;
    }
    $logObj->newEntry(logEntry => $logEntry);
    if ($logObj->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $logObj->errorMessage);
      return undef;
    }
  }

  my $stateObj;
  eval { $stateObj = $stateName->new(langObj => $self->{langObj}, configObj => $self->{configObj}, input => $self->{input},
                                 cookieObj => $self->{cookieObj}, dbHandle => $self->{dbHandle}, logObj => $logObj,
                                 sessionObj => $self->{appSessions}->{apps}->{squidGuard}, userObj => $self->{userObj},
                                 companyObj => $self->{companyObj}, applicationObj => $self->{applicationObj}, appObj => $self->{appObj},
                                 portalSession => $self->{portalSession}, authObj => $self->{authObj}); };
  if ($@)
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{stateEvalFailed}, $stateName, $@));
    if ($self->{firstTime})
    {
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
    }
    return $doc;
  }
  if ($stateObj->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $stateObj->errorMessage);
    return $doc;
  }

  eval { $doc = $stateObj->run(command => $command); };
  if ($@)
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{stateEvalFailed}, $stateName, $@));
    if ($self->{firstTime})
    {
      $self->{appSessions}->{apps}->{squidGuard}->delete;  # remove the session info.
      my $tmpDoc = HTMLObject::Base->new;
      $tmpDoc = $self->{appSessions}->writeCookie(doc => $tmpDoc);  # This will allow the parent session to be properly updated!
    }
    return $doc;
  }
  if ($stateObj->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $stateObj->errorMessage);
    return $doc;
  }
  $self->{appSessions}->writeCookie(doc => $doc);  # make sure the application session id cookie is reset.
  if (ref($doc) =~ /Normal/)
  {
    # we are a Normal document.
    $doc->setJavascriptErrorHandlerEmail(email => $self->{configObj}->{emailAddress});
  }

  return $doc;  # signal all ok.
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

squidGuard - Object used to build a squidGuard Class.

=head1 SYNOPSIS

  use Portal::squidGuard;
  my $obj = Portal::squidGuard->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

squidGuard is the main class for the squidGuard Application.

=head1 Exported FUNCTIONS

  scalar new(langObj, configObj, applicationObj, input, doc, cookieObj,
             appObj, userObj, companyObj, portalSession,
             browserType, browserVersion, browserMaker, authObj)
    Creates a new instance of the Portal::squidGuard
    object.
      langObj        - Portal::Language
      configObj      - Portal::Data::Config
      applicationObj - Portal::Application
      input          - hash of input values
      doc            - place holder for HTMLObject to be defined
      cookieObj      - HTMLObject::ReadCookie
      appObj         - Portal::Objects::ApplicationObject
      userObj        - Portal::Objects::UserObject
      companyObj     - Portal::Objects::CompanyObject
      portalSession  - Portal::Session used to manage all squidGuard Sessions
      browserMaker   - Netscape, IE, Phone.Com, etc.
      browserType    - HTML or HDML or WML
      browserVersion - version to display to
      authObj        - Portal::Auth (no billingDB defined)

  bool isValid(void)
    Runs validation code, returns 0, 1 to indicate if the object is invalid, valid.
    The error will be available via errorMessage().
    
  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.
    
  HTMLObject run(state, command)
    Pulls the specified state file into existance and calls run(command).
    Returns the HTMLObject document that represents the output from the
    App's State->run() method.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC&Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
